using System;
using System.Text;
using System.Data;
using System.Data.OleDb;
using System.Data.Common;
using System.Collections.Specialized;
using System.IO;

namespace My
{
    class ExcelReader
    {
        string fileName = "";
        OleDbConnection connection = null;
        StringCollection workSheets = new StringCollection();
        public string lastExceptionMessage = "";

        public ExcelReader()
        {
            
        }
        
        public ExcelReader(string name)
        {
            this.fileName = name;
        }

        public bool Open()
        {
            return this.Open(fileName);
        }

        public bool Open(string name)
        {
            if (!File.Exists(name))
                return false;
            
            fileName = name;
                
            string connectionString = @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source="+name+@";Extended Properties=""Excel 8.0;HDR=YES;""";
            connection = new OleDbConnection(connectionString);

            try
            {
                connection.Open();
                if (ExtractNamesOfWorksheets())
                    return true;
                else
                    return false;
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                return false;
            }

        }

        public bool ExtractNamesOfWorksheets()
        {
            if (connection.State != ConnectionState.Open)
                return false;
            try
            {
                DataTable sheets = connection.GetSchema("Tables");
                workSheets.Clear();
                foreach (DataRow row in sheets.Rows)
                {
                    workSheets.Add(row["TABLE_NAME"].ToString().Replace("'", ""));
                }
                sheets.Dispose();
                return true;
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                return false;
            }
        }

        public bool GetWorksheetColumnNames(string sheetname, out StringCollection columns)
        {
            DataTable c;
            columns = new StringCollection();

            if (WorksheetExists(sheetname))
                return false;
            
            try
            {
                c = connection.GetSchema("Columns", new string[] { null, null, sheetname, null });

                foreach (DataRow row in c.Rows)
                {
                    columns.Add(row["COLUMN_NAME"].ToString().Replace("'", ""));
                }

                c.Dispose();
                return true;

            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                return false;
            }
        }

        private string AppendDollarSign(string str)
        {
            if (!str.EndsWith("$"))
                str += "$";
            return str;
        }


        public bool WorksheetExists(string name)
        {
            return (workSheets.Contains(AppendDollarSign(name)));
        }

        public DbDataReader GetWorksheetReader(string sheetname)
        {
            string sn = AppendDollarSign(sheetname);
            
            if (!WorksheetExists(sn))
                throw (new Exception(String.Format("Worksheet {0} does not exist!",sheetname)));

            DbCommand command = connection.CreateCommand();
            command.CommandText = "select * from ["+sn+"]";

            try
            {
                return (command.ExecuteReader());
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                throw (new Exception(String.Format("Could not get data reader for Worksheet {0} reason:{1}", sheetname, e.Message))); 
            }
        }


        public void Close()
        {
            if (connection == null)
                return;

            try
            {
                if (connection.State == ConnectionState.Open)
                {
                    connection.Close();
                    connection.Dispose();
                }
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
            }
        }
    
    }
}
