
using System;
using System.Data;
using System.Text;
using System.Data.OleDb;
using System.IO;
using System.Data.Common;
using System.Collections.Specialized;
using System.Collections;

namespace My
{
    public class ExcelWriter
    {
        public string fileName;
        OleDbConnection connection = null;
        public string lastExceptionMessage = "";
        StringCollection workSheets = new StringCollection();
        
        public ExcelWriter()
        {
        
        }
        
        public ExcelWriter(string fn)
        {
            this.fileName = fn;
            if (!CreateNewExcelFile(fileName))
                throw new Exception("Could not create Excel file "+fileName);
        }

        ~ExcelWriter()
        {
            Close();

        }

        public void Close()
        {
            try
            {
                if (connection.State != ConnectionState.Closed)
                    connection.Close();
            }
            catch (Exception ex)
            {
                lastExceptionMessage = ex.Message;
            }
        }


        public bool CreateNewWorksheet(string sheetname, string[] columns)
        {
            if ((sheetname.Trim() == "") || (columns.Length < 1) )
            {
                return false;
            }

            DbCommand command = connection.CreateCommand();
            StringBuilder CommandText = new StringBuilder("CREATE TABLE " + sheetname);


                CommandText.Append(" (");
                foreach (string c in columns)
                {
                    CommandText.Append(" " + c + " char(255),");
                }
                CommandText.Remove(CommandText.Length - 1, 1);

                CommandText.Append(" ) ");

            command.CommandText = CommandText.ToString();

            try
            {
                command.ExecuteNonQuery();
                return true;
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                throw (new Exception(String.Format("Could not get data reader for Worksheet {0} reason:{1}", sheetname, e.Message)));
            }


        }


        public bool InsertRowIntoExistingWorksheet(string sheetname, string[] keys, string[] values)
        {
            if (keys.Length != values.Length)
                return false;

            SortedList sl = new SortedList();
            for (int i = 0; i < keys.Length; i++)
            {
                sl.Add(keys[i],values[i]);
            }
            return InsertRowIntoExistingWorksheet(sheetname, sl);

        }

        public bool InsertRowIntoExistingWorksheet(string sheetname, SortedList keysandvalues)
        {
            DbCommand command = connection.CreateCommand();

            StringBuilder CommandText = new StringBuilder("INSERT INTO [" + AppendDollarSign(sheetname) + "] ");
            StringBuilder Columns = new StringBuilder(" ( ");
            StringBuilder Values = new StringBuilder(" ( ");

            foreach (DictionaryEntry de in keysandvalues)
            {
                Columns.Append(de.Key + ", ");
                Values.Append("'"+de.Value + "', ");
            }

            Columns.Remove(Columns.Length - 2, 2);
            Values.Remove(Values.Length - 2, 2);

            Columns.Append(" ) ");
            Values.Append(" ) ");
            
            command.CommandText = CommandText.ToString() + Columns.ToString() +" VALUES "+ Values.ToString() ;

            try
            {
                command.ExecuteNonQuery();
                return true;
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                throw (new Exception(String.Format("Could not get data reader for Worksheet {0} reason:{1}", sheetname, e.Message)));
            }            
            
        }
        

        public bool InsertDataToWorksheet(string sheetname, DataTable dt)
        {
            ExtractNamesOfWorksheets();

            string[] cols = new string[dt.Columns.Count];
            int i = 0;
            foreach (DataColumn c in dt.Columns)
            {
                cols[i++] = c.ColumnName;
            }
            
            if (!WorksheetExists(sheetname))
            {
                if (!CreateNewWorksheet(sheetname, cols))
                    return false;
            }

            SortedList kv;
            foreach (DataRow dr in dt.Rows)
            {
                kv = new SortedList();
                foreach (string cn in cols)
                    kv.Add(cn, dr[cn]);
                InsertRowIntoExistingWorksheet(sheetname, kv);
            }    
            return true;
        }

        public bool WorksheetExists(string name)
        {
            return (workSheets.Contains(AppendDollarSign(name)));
        }


        public bool ExtractNamesOfWorksheets()
        {
            if (connection.State != ConnectionState.Open)
                return false;
            try
            {
                DataTable sheets = connection.GetSchema("Tables");
                workSheets.Clear();
                foreach (DataRow row in sheets.Rows)
                {
                    workSheets.Add(row["TABLE_NAME"].ToString().Replace("'", ""));
                }
                sheets.Dispose();
                return true;
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                return false;
            }
        }
        
        
        protected bool CreateNewExcelFile(string name)
        {
        
            if (File.Exists(name))
            {
               File.Delete(name);
               if (File.Exists(name))
                   return false;
            }
            
            fileName = name;

            string connectionString = @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source="+name+@";Extended Properties=""Excel 8.0;HDR=YES;""";
            connection = new OleDbConnection(connectionString);

            try
            {
                connection.Open();
                return true;
            }
            catch (Exception e)
            {
                lastExceptionMessage = e.Message;
                return false;
            }

        }

        private string AppendDollarSign(string str)
        {
            if (!str.EndsWith("$"))
                str += "$";
            return str;
        }

        
    
    }
}
